function [errors]=fem_1d_errors(uh,geom,p,Ne,uex,u1ex,varargin)
% FEM_1D_ERRORS Computes errors for 1d b.v.p.
% [errors]=fem_1d_errors(uh,geom,p,Ne,uex,u1ex)
% [errors]=fem_1d_errors(uh,geom,p,Ne,uex,u1ex,errtype)
% [errors]=fem_1d_errors(uh,geom,p,Ne,uex,u1ex,errtype,degree)
%
% Input:
%        uh= numerical solution
%        geom = struct with fields:
%          geom.xa, geom.xb = end-points of Omega
%          p = local polynomial degree (1 or 2)
%          Ne =  number of elements of the partition
%          uex =  function handle @(x) with the exact solution
%          u1ex =  function handle @(x) with the derivative 
%                  of the exact solution
%optional input: errtype =  0 : absolute errors (default value)
%                        =  1 : relative errors
%                degree = degree of exactness of quadrature formulas
%                         (default value =4)
% Output: 
% 
%         errors.h1 = ||u-uh|| with respect to H1-norm
%         errors.l2 = ||u-uh|| with respect to L2-norm
%

%   Written by Paola Gervasio
%   $Date: 2020/04/26$


% set errtype:
%          errtype =  0 : absolute errors
%          errtype =  1 : relative errors

if nargin==6
  errtype=0;
  degree=4;
elseif nargin==7
  errtype=varargin{1};
  degree=4;
else
   errtype=varargin{1};
  degree=varargin{2};
 
end

% ||u_ex-u_h|| with respect to H1 norm

% 
% 
if p==1
N=Ne+1; % number of nodes
vmesh.nodes=linspace(geom.xa,geom.xb,N)'; % nodes
% connectivity matrix
nov=(1:2)'; nmax=2;
for ie=2:Ne
  nov=[nov,(nmax:nmax+1)'];
  nmax=nmax+1;
end
vmesh.nov=nov;
vmesh.ne=Ne;
[basis,jacf,qf]=fem_1d_setting(vmesh,p,degree);
fun_set.uex=uex;
fun_set.uexx=u1ex;
[errors.h1,errors.l2]=normh1_p1_1d(errtype,uh,vmesh,basis,jacf,qf,fun_set);

elseif p==2
N=2*Ne+1; % number of nodes
vmesh.nodes=linspace(geom.xa,geom.xb,N)'; % nodes
% connectivity matrix
nov=(1:3)'; nmax=3;
for ie=2:Ne
  nov=[nov,(nmax:nmax+2)'];
  nmax=nmax+2;
end
vmesh.nov=nov;
vmesh.ne=Ne;
[basis,jacf,qf]=fem_1d_setting(vmesh,p,degree);
fun_set.uex=uex;
fun_set.uexx=u1ex;

[errors.h1,errors.l2]=normh1_p2_1d(errtype,uh,vmesh,basis,jacf,qf,fun_set);
end

end




function [err_h1,err_l2]=normh1_p1_1d(errtype,un,vmesh,basis,jacf,qf,fun_set)
%  NORMH1_P1_1d: compute the H1-norm and L2-norm errors
% for Lagrange P1 fem in 1d domains 
%  [err_h1,err_l2]=normh1_p1_1d(un,vmesh,basis,jacf,qf,fun_set)
% Input : 
%         errtype = 0 for absolute error ||u-u_ex||
%                   1 for relative error ||u-u_ex||/||u_ex||
%         un = numerical solution
%         vmesh = structure with the mesh
%         basis = structure (derivative of the basis functions)
%         jacf = straucture with jacobians at the LGL nodes
%         qf = structure qith jacobians at the quadrature nodes
%         fun_set = stracture with exact solution
%    
% Output: err_h1 =  H1-norm error  
%         err_l2 =  L2-norm error  
%

%   Written by Paola Gervasio
%   $Date: 2018/06/29$

[~,ne]=size(vmesh.nov);

nor_l2_n=0; nor_l2_d=0; nor_h1_n=0; nor_h1_d=0;
X=qf.xq';
nq=length(qf.wq);
for ie=1:ne
    % rimappo i nodi di quadratura sull'elemento ie
novl=vmesh.nov(:,ie);
x=vmesh.nodes(novl,1); 


jac=jacf{ie}.jac;
B=jac;
B1t=jacf{ie}.Jmt;

X1=B*X+x(1)*ones(1,nq);
xq=X1(1,:)';


% valuto la soluzione numerica nei nodi di quadratura
uu=un(novl);
uh_quad=basis.phi*uu;

% valuta la soluzione esatta nei nodi di quadratura
uex_quad=fun_set.uex(xq);

% norma L^2 di (u-u_h)

s0=sum((uex_quad-uh_quad).^2.*qf.wq)*jac;
s0d=sum(uex_quad.^2.*qf.wq)*jac;
nor_l2_n=nor_l2_n+s0;
nor_l2_d=nor_l2_d+s0d;

% norma H^1 di (u-u_h)

% valuto il gradiente di u_h (per i P1 e' un vettore costante sul
% triangolo, quindi indip dal nodo di quadratura)
% grad phi_i=B^{-T} \hat grad (\hat phi_i)
% Per i P1, il gradiente sul singolo elemento e' un vettore costante
grad=[-1,1]; 
gphi=B1t*grad;
grad_uh=gphi*uu;
% valuta il grad della soluzione esatta nei nodi di quadratura
uexx_quad=fun_set.uexx(xq);
sx=sum((uexx_quad-grad_uh(1)).^2.*qf.wq)*jac;
sxd=sum(uexx_quad.^2.*qf.wq)*jac;
nor_h1_n=nor_h1_n+(s0+sx);
nor_h1_d=nor_h1_d+(s0d+sxd);
end

if errtype==0 % absolute error
    err_h1=sqrt(nor_h1_n);
    err_l2=sqrt(nor_l2_n);
elseif errtype==1 % relative error
    if abs(nor_h1_d)>1.d-14; err_h1=sqrt(nor_h1_n/nor_h1_d); 
    else
    err_h1=sqrt(nor_h1_n); 
    end
    if abs(nor_l2_d)>1.d-14; err_l2=sqrt(nor_l2_n/nor_l2_d); 
    else
    err_l2=sqrt(nor_l2_n); 
    end
end
end
function [err_h1,err_l2]=normh1_p2_1d(errtype,un,vmesh,basis,jacf,qf,fun_set)
%  NORMH1_P2_1d: compute the H1-norm and L2-norm error
% for Lagrange P2 fem in 1d domains 
%  [err_h1,err_l2]=normh1_p2_1d(un,vmesh,basis,jacf,qf,fun_set)
% Input : 
%         errtype = 0 for absolute error ||u-u_ex||
%                   1 for relative error ||u-u_ex||/||u_ex||
%         un = numerical solution
%         vmesh = structure with the mesh
%         basis = structure (derivative of the basis functions)
%         jacf = straucture with jacobians at the LGL nodes
%         qf = structure qith jacobians at the quadrature nodes
%         fun_set = stracture with exact solution
%    
% Output: err_h1 =  H1-norm error  
%         err_l2 =  L2-norm error  
%

%   Written by Paola Gervasio
%   $Date: 2018/06/29$

[~,ne]=size(vmesh.nov);

nor_l2_n=0; nor_l2_d=0; nor_h1_n=0; nor_h1_d=0;
X=qf.xq';
nq=length(qf.wq);
for ie=1:ne
    % rimappo i nodi di quadratura sull'elemento ie
novl=vmesh.nov(:,ie);
x=vmesh.nodes(novl,1); 


jac=jacf{ie}.jac;
B=jac;
B1t=jacf{ie}.Jmt;


X1=B*X+x(1)*ones(1,nq);
xq=X1(1,:)';


% valuto la soluzione numerica nei nodi di quadratura
uu=un(novl);
uh_quad=basis.phi*uu;

% valuta la soluzione esatta nei nodi di quadratura
uex_quad=fun_set.uex(xq);

% norma L^2 di (u-u_h)

s0=sum((uex_quad-uh_quad).^2.*qf.wq)*jac;
s0d=sum(uex_quad.^2.*qf.wq)*jac;
nor_l2_n=nor_l2_n+s0;
nor_l2_d=nor_l2_d+s0d;

% norma H^1 di (u-u_h)

% valuto il gradiente di u_h 
% grad phi_i=B^{-T} \hat grad (\hat phi_i)
uhx=zeros(nq,1);

for q=1:nq
gphi1=B1t*basis.phix(q,:)*uu;     
uhx(q)=gphi1(1);
end

% valuta il grad della soluzione esatta nei nodi di quadratura
uexx_quad=fun_set.uexx(xq);
sx=sum((uexx_quad-uhx).^2.*qf.wq)*jac;
sxd=sum(uexx_quad.^2.*qf.wq)*jac;
nor_h1_n=nor_h1_n+(s0+sx);
nor_h1_d=nor_h1_d+(s0d+sxd);

end

if errtype==0 % absolute error
    err_h1=sqrt(nor_h1_n);
    err_l2=sqrt(nor_l2_n);
elseif errtype==1 % relative error
    if abs(nor_h1_d)>1.d-14; err_h1=sqrt(nor_h1_n/nor_h1_d); 
    else
    err_h1=sqrt(nor_h1_n); 
    end
    if abs(nor_l2_d)>1.d-14; err_l2=sqrt(nor_l2_n/nor_l2_d); 
    else
    err_l2=sqrt(nor_l2_n); 
    end
end

end