function [basis,jacf,qf]=fem_1d_setting(vmesh,p,degree,varargin)
% FEM_1D_SETTING: functions, jacobians, normal versors
% this function works on all the elements of the domain
%  [basis,jacf,qf,boundary]=fem_1d_setting(vmesh,param)
% Input: vmesh = structure with the mesh on the domain
%        degree: precision degree of quadrature formulas 
%        varargin{1}= qftype: 0 = Legendre-Gauss qf, 
%                             1 = Legendre-Gauss-Lobatto qf 
% Output: 
%         basis.phi phi_j(x_q) Lagrange basis at the quadrature nodes
%         basis.dphix dphi_j/dx(x_q) dx of the Lagrange basis at quad nodes
%         jacf{ie}, with ie=1,...ne structure with fields:
%                 .jac = det(J_F_ie) determinant of the Jacobian of the map
%         qf.xq, .wq quadrature nodes and weights on the reference element (0,1)
%         qf.xq_ph, .wq_ph quadrature nodes and weights on all elements
% 
% needed functions: all in Level_0 directory

%   Written by Paola Gervasio
%   $Date: 2019/04/11$

if nargin==3
    qftype=0;
else
    qftype=varargin{1};

end
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% compute quadrature nodes and weights in the 1d reference simplex
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

np=1+ceil((degree-1)/2);    
[qf]=quadrature_gauss_1d(np,qftype);
qf.xq=remap_1d(qf.xq,[0,1]);
qf.wq=qf.wq/2;
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% basis functions and its derivatives at the quadrature nodes in the
% reference simplex
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if p==1 %  P1 in [0,1]
  [basis]=basis_p1_1d(qf);
elseif p==2 % P2  in [0,1]
  [basis]=basis_p2_1d(qf);
end


% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% basis functions and derivatives at the quadrature nodes in the
% Jacobian matrices
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
jacf=cell(vmesh.ne,1);

for ie=1:vmesh.ne
nov_l=vmesh.nov(:,ie);
jacf{ie}.jac=abs(vmesh.nodes(nov_l(end))-vmesh.nodes(nov_l(1)));
jacf{ie}.Jmt=1/(jacf{ie}.jac);
jacf{ie}.jacx= jacf{ie}.jac;
jacf{ie}.coefx= 1;
jacf{ie}.trax=vmesh.nodes(nov_l(1));
end

% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% compute quadrature nodes and weights in each element of the mesh
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
[nq,~]=size(qf.xq);
ne=vmesh.ne;
qf.xq_ph=zeros(nq,ne);
qf.wq_ph=zeros(nq,ne);

for ie=1:vmesh.ne
xq=qf.xq*jacf{ie}.jacx+jacf{ie}.trax;
jac=jacf{ie}.jac;

qf.xq_ph(:,ie)=xq;
qf.wq_ph(:,ie)=qf.wq*jac;

end

end
function [qf]=quadrature_gauss_1d(np,type)
% quadrature_guass_1d : quadrature nodes and weights 
% [qf]=quadrature_gauss_1d(np,type)
% Input: degree= precision degree of the quadrature formula
%        type = 0 LG formulas,
%             ~=0 LGL formulas
% Ouput: qf.xq (coordinates of the nodes)
%        qf.wq weights


if type==0
% LG formulas
[qf.xq,qf.wq]=xwlg(np);
else
% LGL formulas
[qf.xq,qf.wq]=xwlgl(np);
end

end

function [basis]=basis_p1_1d(qf)
% basis_p1_1d: computes quadrature nodes and weights and evaluate there 
% the P1-basis on the reference interval (0,1)
% [basis]=basis_p1_1d(degree)
%
% Input: qf structure with quadrature nodes
% Output:
%        basis.phi = Lagrange basis functions evaluated at the quadrature nodes 
%        basis.phix   = Lagrange basis functions derivatives 
%                        evaluated at the quadrature nodes 
%        basis.phi_wq = array of values:  phi_i(x_q) * w_q
%        basis.phi_ij_wq = array of values: phi_j(x_q) * phi_i(x_q) * w_q
%        basis.mat00_{ij}=(phi_j,phi_i)_T
%        basis.mat11_{ij}=(dphi_j/dx,dphi_i/dx)_T
 
%   Written by Paola Gervasio
%   $Date: 2015/04/15$
%   $Date: 2019/04/27$

nq=length(qf.xq);
zz=zeros(nq,1); % vettore nullo
uno=ones(nq,1);

% basis functions 
basis.phi=[1-qf.xq, qf.xq];
[~,nb]=size(basis.phi);
% derivates:
% dphi/dx
basis.phix=[-uno,uno];

% phi *wq
basis.phi_wq=zeros(nq,nb);
for i=1:nb
basis.phi_wq(:,i)=basis.phi(:,i).*qf.wq;
end

% phi *wq
basis.phi_ij_wq=zeros(nq,nb,nb);

for i=1:nb
for j=1:nb
basis.phi_ij_wq(:,j,i)=basis.phi(:,i).*basis.phi_wq(:,j);
end
end

% mat00_{ij}=(phi_j,phi_i)_T
basis.mat00=[ 2,1; 1,2 ]/6;
% mat1_{ij}=dphi_j/dx_i
basis.mat1=[ ]; 

 

  % mat11_{ij}=(dphi_j/dx,dphi_i/dx)_T
basis.mat11=[1,-1;-1,1];

end

function [basis]=basis_p2_1d(qf)
% basis_p2_1d: computes quadrature nodes and weights and evaluate there 
% the P2-basis on the reference interval (0,1)
% [basis]=basis_p2_1d(degree)
%
% Input: qf structure with quadrature nodes
% Output:
%        basis.phi = Lagrange basis functions evaluated at the quadrature nodes 
%        basis.phix   = Lagrange basis functions derivatives 
%                        evaluated at the quadrature nodes 
%        basis.phi_wq = array of values:  phi_i(x_q) * w_q
%        basis.phi_ij_wq = array of values: phi_j(x_q) * phi_i(x_q) * w_q
%        basis.mat00_{ij}=(phi_j,phi_i)_T
%        basis.mat11_{ij}=(dphi_j/dx,dphi_i/dx)_T
 
%   Written by Paola Gervasio
%   $Date: 2015/04/15$
%   $Date: 2019/04/27$

nq=length(qf.xq);
xq=qf.xq;
 

% basis functions 
basis.phi=[2*xq.^2-3*xq+1, -4*xq.^2+4*xq, 2*xq.^2-xq];


[~,nb]=size(basis.phi);
% derivates:
% dphi/dx
basis.phix=[4*xq-3,-8*xq+4,4*xq-1];
 

% phi *wq
basis.phi_wq=zeros(nq,nb);
for i=1:nb
basis.phi_wq(:,i)=basis.phi(:,i).*qf.wq;
end

% phi *wq
basis.phi_ij_wq=zeros(nq,nb,nb);

for i=1:nb
for j=1:nb
basis.phi_ij_wq(:,j,i)=basis.phi(:,i).*basis.phi_wq(:,j);
end
end

% mat00_{ij}=(phi_j,phi_i)_T
basis.mat00=[4, 2, -1;
        2, 16, 2;
       -1, 2,  4]/30;
 % mat1_{ij}=dphi_j/dx_i
basis.mat1=[ ]; 

 

  % mat11_{ij}=(dphi_j/dx,dphi_i/dx)_T
basis.mat11=[7, -8, 1;
             -8,16,-8;
             1,-8,7]/3; 

end
function [qx1]=remap_1d(qx,xx);
% REMAP_1D: remap some nodes from [-1,1]  into the interval with vertices
% in xx 
% Input : qx = 1-column array with the coordinate of the nodes to remap
%         xx = array with x of the 2 endpoints of the nterval
% Ouput: qx1 = 1-column array with the coordinate of the remapped nodes.


xm=(xx(2)-xx(1))/2; xp=(xx(2)+xx(1))/2;

qx1(:,1)=qx(:,1)*xm+xp;
end
